/*
 * Copyright (c) 2024-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include <ostream>
#include <vector>

class IBNode;
class IBDiag;

class SimInfoData {
    public:
        static const std::vector<const char*>  SMPCapMaskBits;
        static const std::vector<const char*>  GMPCapMaskBits;
        static const std::vector<const char*>  PortInfoCapMaskBits;
        static const std::vector<const char*>  PortInfoCapMask2Bits;
};

class SimInfoDumpPy {
    public:
        IBDiag & m_ibdiag;

    public:
        SimInfoDumpPy(IBDiag & ibdiag)
            : m_ibdiag(ibdiag)
        {}

        int Dump(const std::string & filename);

    private:
        void DumpPortInfoAllCapMask(std::ostream & sout, IBNode & node);
        void DumpPortInfoCapMask(std::ostream & sout, phys_port_t index, SMP_PortInfo *p_info, bool is_cap_mask2);

    private:
        void DumpSMPCapMask(std::ostream & sout, IBNode & node);
        void DumpGMPCapMask(std::ostream & sout, IBNode & node);
};
