 /*
 * Copyright (c) 2004-2021 Mellanox Technologies LTD. All rights reserved.
 * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _DIAGNOSTIC_DATA_H_
#define _DIAGNOSTIC_DATA_H_

#include <infiniband/ibdiag/ibdiagnet/ibdiagnet_plugins_ifc.h>
#include "../internal_packets/internal_packets_layouts.h"
#include "phy_diag_types.h"
#include "infiniband/ibdm/cable/PhyCableRecord.h"
#include "phy_export_api.h"

#define DIAGNOSTIC_DATA_ALL_NAME                        "dd_all"

#define DIAGNOSTIC_DATA_PAGE_IDENTIFICATION_PAGE        31
#define DIAGNOSTIC_DATA_PAGE_IDENTIFICATION_VERSION     1
#define DIAGNOSTIC_DATA_PAGE_IDENTIFICATION_NUM_FIELDS  8
#define DIAGNOSTIC_DATA_PAGE_IDENTIFICATION_NAME        "dd_pgid"


#define DIAGNOSTIC_DATA_PHYS_LAYER_CNTRS_PAGE       254
#define DIAGNOSTIC_DATA_PHYS_LAYER_CNTRS_VERSION     1
#define DIAGNOSTIC_DATA_PHYS_LAYER_CNTRS_NUM_FIELDS  26
#define DIAGNOSTIC_DATA_PHYS_LAYER_CNTRS_NAME        "dd_ppcnt_plc"

#define DIAGNOSTIC_DATA_PCIE_CNTRS_PAGE           2
#define DIAGNOSTIC_DATA_PCIE_CNTRS_VERSION        1
#define DIAGNOSTIC_DATA_PCIE_CNTRS_NUM_FIELDS     20
#define DIAGNOSTIC_DATA_PCIE_CNTRS_NAME           "dd_mpcnt_pci_cnt"

#define DIAGNOSTIC_DATA_PCIE_LANES_PAGE           3
#define DIAGNOSTIC_DATA_PCIE_LANES_VERSION        1
#define DIAGNOSTIC_DATA_PCIE_LANES_NUM_FIELDS     18
#define DIAGNOSTIC_DATA_PCIE_LANES_NAME           "dd_mpcnt_pci_lcnt"

#define DIAGNOSTIC_DATA_PCIE_TIMERS_PAGE          4
#define DIAGNOSTIC_DATA_PCIE_TIMERS_VERSION       1
#define DIAGNOSTIC_DATA_PCIE_TIMERS_NUM_FIELDS    23
#define DIAGNOSTIC_DATA_PCIE_TIMERS_NAME          "dd_mpcnt_pci_timers"

// PDDR pages
#define DIAGNOSTIC_DATA_TROUBLESHOOTING_INFO_PAGE        253
#define DIAGNOSTIC_DATA_TROUBLESHOOTING_INFO_VERSION     1
#define DIAGNOSTIC_DATA_TROUBLESHOOTING_INFO_NUM_FIELDS  5
#define DIAGNOSTIC_DATA_TROUBLESHOOTING_INFO_NAME        "dd_pddr_ti"

#define DIAGNOSTIC_DATA_OPERATION_INFO_PAGE        252
#define DIAGNOSTIC_DATA_OPERATION_INFO_VERSION     1
#define DIAGNOSTIC_DATA_OPERATION_INFO_NUM_FIELDS  31
#define DIAGNOSTIC_DATA_OPERATION_INFO_NAME        "dd_pddr_op"

#define DIAGNOSTIC_DATA_PHY_INFO_PAGE        251
#define DIAGNOSTIC_DATA_PHY_INFO_VERSION     1
#define DIAGNOSTIC_DATA_PHY_INFO_NUM_FIELDS  ACC_REG_PDDR_PHY_INFO_FIELDS_NUM
#define DIAGNOSTIC_DATA_PHY_INFO_NAME        "dd_pddr_phy"

#define DIAGNOSTIC_DATA_MODULE_INFO_PAGE        250
#define DIAGNOSTIC_DATA_MODULE_INFO_VERSION     1
#define DIAGNOSTIC_DATA_MODULE_INFO_NUM_FIELDS  100
#define DIAGNOSTIC_DATA_MODULE_INFO_NAME        "dd_pddr_module"

#define DIAGNOSTIC_DATA_LINK_DOWN_INFO_PAGE        248
#define DIAGNOSTIC_DATA_LINK_DOWN_INFO_VERSION     1
#define DIAGNOSTIC_DATA_LINK_DOWN_INFO_NUM_FIELDS  38
#define DIAGNOSTIC_DATA_LINK_DOWN_INFO_NAME        "dd_pddr_ldown"

#define DIAGNOSTIC_DATA_LINK_UP_INFO_PAGE        242
#define DIAGNOSTIC_DATA_LINK_UP_INFO_VERSION     1
#define DIAGNOSTIC_DATA_LINK_UP_INFO_NUM_FIELDS  ACC_REG_PDDR_LINKUP_FIELDS_NUM
#define DIAGNOSTIC_DATA_LINK_UP_INFO_NAME        "dd_pddr_lup"

#define DIAGNOSTIC_DATA_LATCHED_FLAG_INFO_PAGE          243
#define DIAGNOSTIC_DATA_LATCHED_FLAG_INFO_VERSION       1
#define DIAGNOSTIC_DATA_LATCHED_FLAG_INFO_NUM_FIELDS    24
#define DIAGNOSTIC_DATA_LATCHED_FLAG_INFO_NAME          "dd_pddr_lfi"

// PPCNT pages
#define DIAGNOSTIC_DATA_RS_FEC_HISTOGRAM_PAGE        247
#define DIAGNOSTIC_DATA_RS_FEC_HISTOGRAM_VERSION     1
#define DIAGNOSTIC_DATA_RS_FEC_HISTOGRAM_NUM_FIELDS  20
#define DIAGNOSTIC_DATA_RS_FEC_HISTOGRAM_NAME        "dd_ppcnt_rsfec"

#define DIAGNOSTIC_DATA_PLR_COUNTERS_PAGE        246
#define DIAGNOSTIC_DATA_PLR_COUNTERS_VERSION     1
#define DIAGNOSTIC_DATA_PLR_COUNTERS_NUM_FIELDS  10
#define DIAGNOSTIC_DATA_PLR_COUNTERS_NAME        "dd_ppcnt_plr"

#define DIAGNOSTIC_DATA_PHY_LAYER_STATISTICS_PAGE        245
#define DIAGNOSTIC_DATA_PHY_LAYER_STATISTICS_VERSION     1
#define DIAGNOSTIC_DATA_PHY_LAYER_STATISTICS_NUM_FIELDS  35
#define DIAGNOSTIC_DATA_PHY_LAYER_STATISTICS_NAME        "dd_ppcnt_plsc"

#define DIAGNOSTIC_DATA_INFINIBAND_GENERAL_COUNTERS_PAGE        241
#define DIAGNOSTIC_DATA_INFINIBAND_GENERAL_COUNTERS_VERSION     1
#define DIAGNOSTIC_DATA_INFINIBAND_GENERAL_COUNTERS_NUM_FIELDS  10
#define DIAGNOSTIC_DATA_INFINIBAND_GENERAL_COUNTERS_NAME        "dd_ppcnt_gen_counters"

#define DIAGNOSTIC_DATA_PACKET_TYPES_COUNTERS_PAGE        240
#define DIAGNOSTIC_DATA_PACKET_TYPES_COUNTERS_VERSION     1
#define DIAGNOSTIC_DATA_PACKET_TYPES_COUNTERS_NUM_FIELDS  6
#define DIAGNOSTIC_DATA_PACKET_TYPES_COUNTERS_NAME        "dd_ppcnt_ptc"


/******************************************************/
/******************************************************/
/******************************************************/
class DiagnosticDataInfo {
private:


protected:
    u_int32_t         m_page_id;
    u_int32_t         m_support_version;
    u_int32_t         m_num_fields;
    u_int64_t         m_not_supported_bit;
    u_int32_t         m_dd_type;
    string            m_header;
    u_int8_t          m_is_per_node;
    PhyPluginSupportedNodesType m_support_nodes;
    IBDiag            *m_p_ibdiag;
    string            m_name;
    string            m_common_header;  
    bool              m_enable_disconnected_ports;

public:
    DiagnosticDataInfo(int page_id, int support_version,
                       int num_fields, const string &name, u_int64_t not_supported_bit,
                       u_int32_t dd_type, const string& header, u_int8_t is_per_node,
                       PhyPluginSupportedNodesType support_nodes = SUPPORT_ALL,
                       bool m_enable_disconnected_ports = false, 
                       const string& common_header = "NodeGuid,PortGuid,PortNum,Version");

    virtual ~DiagnosticDataInfo() {}

    string GetSectionHeader() const { return m_header; }
    inline const string& GetName() const { return this->m_name; }
    inline int GetPageId() const { return m_page_id; }
    inline int GetSupportedVersion() const { return m_support_version; }
    inline u_int64_t GetNotSupportedBit() const { return m_not_supported_bit; }
    inline void SetIBDiag(IBDiag *p_ibdiag) { this->m_p_ibdiag = p_ibdiag; }
    inline u_int32_t GetDDType() const { return m_dd_type; }
    inline u_int8_t IsPerNode() const { return m_is_per_node; }

    inline bool IsDDPageSupportNodeType(const IBNode* node);

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const;
    void DumpDiagnosticDataHeaderEnd(CSVOut &csv_out) const;

    bool IsDDPageSupportedInNode(struct DDPageIdentification *p_page_identification);
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const = 0;

    inline bool EnableDisconnectedPorts() { return this->m_enable_disconnected_ports; }
    inline void SetEnableDisconnectedPorts(bool enable_disconnected_ports) {
        this->m_enable_disconnected_ports = enable_disconnected_ports;
    }

    virtual void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                                      export_data_phy_node_t *p_export_data_phy_node,
                                      VS_DiagnosticData &dd) const {}
};

template<typename DataStruct>
DataStruct *ExportDiagnosticDataImpl(void (*unpack_func)(DataStruct *, const u_int8_t *),
                                     VS_DiagnosticData &dd)
{
    DataStruct *p_data_struct = new DataStruct;
    unpack_func(p_data_struct, dd.data_set.gen_set.data);
    return p_data_struct;
}

bool DiagnosticDataInfo::IsDDPageSupportNodeType(const IBNode* node)
{
    IBDIAGNET_ENTER;
    bool rc = IsPhyPluginSupportNodeType(m_support_nodes, node);
    IBDIAGNET_RETURN(rc);
}


class DiagnosticDataPageIdentification : public DiagnosticDataInfo {
public:
    DiagnosticDataPageIdentification();
    virtual ~DiagnosticDataPageIdentification() {};

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_node)
            p_export_data_phy_node->p_dd_page_identification =
                ExportDiagnosticDataImpl(DDPageIdentification_unpack, dd);
    }
};


/*** Diagnostic Data PCI ***/
class DiagnosticDataPCI : public DiagnosticDataInfo {
public:
    DiagnosticDataPCI(int page_id, int support_version,
            int num_fields, const string &name, u_int64_t not_supported_bit,
            u_int32_t dd_type, string header, u_int8_t is_per_node,
            PhyPluginSupportedNodesType support_nodes = SUPPORT_ALL);

    virtual ~DiagnosticDataPCI() {}

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const = 0;
};

class DiagnosticDataPCIECntrs : public DiagnosticDataPCI {
public:
    DiagnosticDataPCIECntrs();
    virtual ~DiagnosticDataPCIECntrs() {}

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_node)
            p_export_data_phy_node->p_dd_pci_perf_counters =
                ExportDiagnosticDataImpl(pcie_perf_counters_unpack, dd);
    }
};

class DiagnosticDataPCIETimers : public DiagnosticDataPCI {
public:
    DiagnosticDataPCIETimers();
    virtual ~DiagnosticDataPCIETimers() {}

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_node)
            p_export_data_phy_node->p_dd_pcie_timers_states =
                ExportDiagnosticDataImpl(pcie_timers_states_unpack, dd);
    }
};

class DiagnosticDataPCIELanes : public DiagnosticDataPCI {
public:
    DiagnosticDataPCIELanes();
    virtual ~DiagnosticDataPCIELanes() {}

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_node)
            p_export_data_phy_node->p_dd_pcie_lanes_counters =
                ExportDiagnosticDataImpl(pcie_lanes_counters_unpack, dd);
    }
};


/*** Diagnostic Data PHY ***/
class DiagnosticDataPhysLayerCntrs : public DiagnosticDataInfo {
public:
    DiagnosticDataPhysLayerCntrs();
    virtual ~DiagnosticDataPhysLayerCntrs() {};

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_phys_counters =
                ExportDiagnosticDataImpl(phys_layer_cntrs_unpack, dd);
    }
};

// PDDR Troubleshooting Info
class DiagnosticDataTroubleshootingInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataTroubleshootingInfo();
    virtual ~DiagnosticDataTroubleshootingInfo() {};

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
                                    
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    static void DumpTroubleshootingInfoRawData(stringstream &sstream, const struct pddr_troubleshooting_page &troubleshooting_info);

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_trouble_shooting_info =
                ExportDiagnosticDataImpl(pddr_troubleshooting_page_unpack, dd);
    }
};
// PDDR Operation Info
class DiagnosticDataOperationInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataOperationInfo();
    virtual ~DiagnosticDataOperationInfo() {};

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    static void DumpOperationInfoRawData(stringstream &sstream, const struct pddr_operation_info_page &operation_info);
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_operation_info =
                ExportDiagnosticDataImpl(pddr_operation_info_page_unpack, dd);
    }
};
// PDDR Phy Info
class DiagnosticDataPhyInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataPhyInfo();
    virtual ~DiagnosticDataPhyInfo() {};

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_phy_info =
                ExportDiagnosticDataImpl(pddr_phy_info_page_unpack, dd);
    }
};
// PDDR Module Info

class DiagnosticDataModuleInfo : public DiagnosticDataInfo {
private:
    static string transmitter_technology_arr[];
    static string ib_compliance_code_arr[];

public:

    DiagnosticDataModuleInfo(bool enable_disconnected_ports);
    virtual ~DiagnosticDataModuleInfo() {};

    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;
    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    static void DumpModuleInfoRawData(stringstream &sstream, const struct DDModuleInfo &module_info);

    static string ConvertCableTemperatureToStr(const struct DDModuleInfo &module_info, u_int16_t temp);

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_module_info =
                ExportDiagnosticDataImpl(DDModuleInfo_unpack, dd);
    }

    static string ConvertCableTemperatureToStr(struct DDModuleInfo &module_info, u_int16_t temp);
    static string ConvertVoltageToStr(u_int16_t vcc);
    static string ConvertMaxPowerToStr(const struct DDModuleInfo &module_info);
    static const string& ConvertTransmitterTechnologyToStr(u_int8_t transmitter_technology);
    static string ConvertCableTechnologyBitsToStr(const struct DDModuleInfo &module_info);
    static string ConvertCableIdentifierToStr(const struct DDModuleInfo &module_info);
    static string ConvertCableLengthToStr(const struct DDModuleInfo &module_info);
    static string ConvertCableLengthSMFiberToStr(const struct DDModuleInfo &module_info);
    static string ConvertCableLengthOMXToStr(const struct DDModuleInfo &module_info, u_int8_t om_index);
    static bool IsCMISCable(uint8_t identifier);
    static string ConvertIBComplianceCodeToStr(u_int8_t ib_compliance_code);
    static u_int8_t SelectNominalBR(const struct DDModuleInfo &module_info);
    static u_int8_t SelectTransmitterTechnology(const struct DDModuleInfo &module_info);
    static bool IsModule(const struct DDModuleInfo &module_info);
    static bool IsActiveCable(const struct DDModuleInfo &module_info);
    static bool IsPassiveCable(const struct DDModuleInfo &module_info);
    static string ConvertCDREnableTxRxToStr(const struct DDModuleInfo &module_info, bool is_rx,
                                            bool is_csv);
    static string ConvertTxEQRxAMPRxEMPToStr(const struct DDModuleInfo &module_info, u_int8_t val,
                                             bool is_csv);
    static string ConvertAttenuationToStr(const struct DDModuleInfo &module_info, bool is_csv);
    static string ConvertFWVersionToStr(const struct DDModuleInfo &module_info, bool is_csv);
    static string ConvertDateCodeToStr(const struct DDModuleInfo &module_info);
    static string ConvertRevisionToStr(const struct DDModuleInfo &module_info);
    static double MWToDBM(double mW);
    static void DumpModuleInfoHeader(stringstream &sstream);
    static void DumpModuleInfoData(stringstream &sstream, VS_DiagnosticData *dd);
    static void DumpModuleInfoData(stringstream &sstream, const struct DDModuleInfo *p_module_info,
                                   u_int16_t module_info_ext);
    static void DumpModuleInfoData(ofstream &sout, VS_DiagnosticData *dd);
    static void DumpModuleInfoData(ofstream &sout, const struct DDModuleInfo *p_module_info,
                                   u_int16_t module_info_ext);
    static PhyCableRecord::ModuleRecord *CreateRecord(const VS_DiagnosticData *p_dd);
    static PhyCableRecord::ModuleRecord *CreateRecord(const struct DDModuleInfo *p_module_info,
                                                      u_int16_t module_info_ext);
    static void CopyTo(const DDModuleInfo &from, PhyCableRecord::ModuleRecord &to);
};
// PDDR Link Down Info
class DiagnosticDataLinkDownInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataLinkDownInfo();
    virtual ~DiagnosticDataLinkDownInfo() {};

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_link_down_info =
                ExportDiagnosticDataImpl(pddr_link_down_info_page_unpack, dd);
    }
};


// PDDR Link Down Info
class DiagnosticDataLinkUpInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataLinkUpInfo();
    virtual ~DiagnosticDataLinkUpInfo() {};

    virtual int DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_link_up_info =
                ExportDiagnosticDataImpl(pddr_link_up_info_page_unpack, dd);
    }
};

// PDDR Latched Flag Info
class DiagnosticDataLatchedFlagInfo : public DiagnosticDataInfo {
public:
    DiagnosticDataLatchedFlagInfo(bool enable_disconnected_ports);
    virtual ~DiagnosticDataLatchedFlagInfo() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    static void DumpLatchedFlagInfoRawData(stringstream &sstream, const struct DDLatchedFlagInfo &latched_flag_info);

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_latched_flag_info =
                ExportDiagnosticDataImpl(DDLatchedFlagInfo_unpack, dd);
    }

    static void DumpLatchedFlagInfoHeader(stringstream &sstream);
    static void DumpLatchedFlagInfoData(stringstream &sstream, VS_DiagnosticData *dd);
    static void DumpLatchedFlagInfoData(stringstream &sstream, const struct DDLatchedFlagInfo *p_latched_flag_info);
    static void DumpLatchedFlagInfoData(ofstream &sout, VS_DiagnosticData *dd);
    static void DumpLatchedFlagInfoData(ofstream &sout, const struct DDLatchedFlagInfo *p_latched_flag_info);
    static PhyCableRecord::LatchedRecord *CreateRecord(const VS_DiagnosticData *p_dd);
    static PhyCableRecord::LatchedRecord *CreateRecord(const struct DDLatchedFlagInfo *p_latched_flag_info);
    static void CopyTo(const DDLatchedFlagInfo &from , PhyCableRecord::LatchedRecord &to);
};


// PPCNT pages
class DiagnosticDataRSHistograms : public DiagnosticDataInfo {
public:
    DiagnosticDataRSHistograms();
    virtual ~DiagnosticDataRSHistograms() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_rs_histograms =
                ExportDiagnosticDataImpl(ppcnt_rs_fec_histograms_counters_unpack, dd);
    }
};

class DiagnosticDataPLRCounters : public DiagnosticDataInfo {
public:
    DiagnosticDataPLRCounters();
    virtual ~DiagnosticDataPLRCounters() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_plr_counters =
                ExportDiagnosticDataImpl(ppcnt_plr_counters_unpack, dd);
    }
};

class DiagnosticDataPhyStatistics : public DiagnosticDataInfo {
public:
    DiagnosticDataPhyStatistics();
    virtual ~DiagnosticDataPhyStatistics() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_dd_phy_statistical_counters =
                ExportDiagnosticDataImpl(DD_PhyStatisticalCounters_unpack, dd);
    }
};

class DiagnosticDataInfiniBandGeneralCounters : public DiagnosticDataInfo {
public:
    DiagnosticDataInfiniBandGeneralCounters();
    virtual ~DiagnosticDataInfiniBandGeneralCounters() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;
    virtual void DumpDiagnosticData(stringstream &sstream, VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_infiniband_general_counters =
                ExportDiagnosticDataImpl(ppcnt_infiniband_general_counter_unpack, dd);
    }
};

class DiagnosticDataPacketTypesCounters : public DiagnosticDataInfo {
public:
    DiagnosticDataPacketTypesCounters();
    virtual ~DiagnosticDataPacketTypesCounters() {};

    virtual int  DumpDiagnosticDataHeaderStart(CSVOut &csv_out) const override;

    virtual void DumpDiagnosticData(stringstream &sstream,
                                    VS_DiagnosticData &dd,
                                    IBNode *p_node = NULL) const override;

    void ExportDiagnosticData(export_data_phy_port_t *p_export_data_phy_port,
                              export_data_phy_node_t *p_export_data_phy_node,
                              VS_DiagnosticData &dd) const override
    {
        if (p_export_data_phy_port)
            p_export_data_phy_port->p_packet_type_counter =
                ExportDiagnosticDataImpl(ppcnt_packet_type_counter_unpack, dd);
    }
};


#endif // _DIAGNOSTIC_DATA_H_
